@@ -35,34 +35,36 @@
 
 #include <libxfce4ui/libxfce4ui.h>
 #include <math.h>
-#include "xfce4++/util.h"
+#include "../xfce4cpp/connections.hh"
 
-using xfce4::PluginSize;
+#ifdef HAVE_MALLOC_TRIM
+#include <malloc.h>
+#endif
+
+using xfce4::PluginShape;
 using xfce4::Propagation;
 using xfce4::TooltipTime;
 
+using namespace std;
+
 /* vim: !sort -k3 */
-static void          about_cb       ();
-static Propagation   command_cb     (GdkEventButton *event, const Ptr<CPUGraph> &base);
-static void          create_bars    (const Ptr<CPUGraph> &base, GtkOrientation orientation);
-static Ptr<CPUGraph> create_gui     (XfcePanelPlugin *plugin);
-static void          delete_bars    (const Ptr<CPUGraph> &base);
-static Propagation   draw_area_cb   (cairo_t *cr, const Ptr<CPUGraph> &base);
-static Propagation   draw_bars_cb   (cairo_t *cr, const Ptr<CPUGraph> &base);
-static void          mode_cb        (XfcePanelPlugin *plugin, const Ptr<CPUGraph> &base);
-static guint         nb_bars        (const Ptr<const CPUGraph> &base);
-static void          set_bars_size  (const Ptr<CPUGraph> &base);
-static void          shutdown       (const Ptr<CPUGraph> &base);
-static PluginSize    size_cb        (XfcePanelPlugin *plugin, guint size, const Ptr<CPUGraph> &base);
-static TooltipTime   tooltip_cb     (GtkTooltip *tooltip, const Ptr<CPUGraph> &base);
-static void          update_tooltip (const Ptr<CPUGraph> &base);
+static void                 about_cb       ();
+static Propagation          command_cb     (GdkEventButton *event, const shared_ptr<CPUGraph> &base);
+static shared_ptr<CPUGraph> create_gui     (XfcePanelPlugin *plugin);
+static Propagation          draw_area_cb   (cairo_t *cr, const shared_ptr<CPUGraph> &base);
+static Propagation          draw_bars_cb   (cairo_t *cr, const shared_ptr<CPUGraph> &base);
+static void                 mode_cb        (XfcePanelPlugin *plugin, const shared_ptr<CPUGraph> &base);
+static void                 shutdown       (const shared_ptr<CPUGraph> &base);
+static PluginShape          size_cb        (XfcePanelPlugin *plugin, guint size, const shared_ptr<CPUGraph> &base);
+static TooltipTime          tooltip_cb     (GtkTooltip *tooltip, const shared_ptr<CPUGraph> &base);
+static void                 update_tooltip (const shared_ptr<CPUGraph> &base, bool force);
 
 void
 cpugraph_construct (XfcePanelPlugin *plugin)
 {
     xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
-    Ptr<CPUGraph> base = create_gui (plugin);
+    auto base = create_gui (plugin);
 
     Settings::init (plugin, base);
 
@@ -71,47 +73,57 @@ cpugraph_construct (XfcePanelPlugin *plugin)
     xfce_panel_plugin_menu_show_about (plugin);
     xfce_panel_plugin_menu_show_configure (plugin);
 
-    xfce4::connect_about           (plugin, [base](XfcePanelPlugin *p) { about_cb(); });
-    xfce4::connect_free_data       (plugin, [base](XfcePanelPlugin *p) { shutdown(base); });
-    xfce4::connect_save            (plugin, [base](XfcePanelPlugin *p) { Settings::write(p, base); });
-    xfce4::connect_configure_plugin(plugin, [base](XfcePanelPlugin *p) { create_options(p, base); });
-    xfce4::connect_mode_changed    (plugin, [base](XfcePanelPlugin *p, XfcePanelPluginMode mode) { mode_cb(p, base); });
-    xfce4::connect_size_changed    (plugin, [base](XfcePanelPlugin *p, guint size) { return size_cb(p, size, base); });
+    xfce4::connect_about           (plugin, [base](XfcePanelPlugin *p) { about_cb (); });
+    xfce4::connect_free_data       (plugin, [base](XfcePanelPlugin *p) { shutdown (base); });
+    xfce4::connect_save            (plugin, [base](XfcePanelPlugin *p) { Settings::write (p, base); });
+    xfce4::connect_configure_plugin(plugin, [base](XfcePanelPlugin *p) { create_options (p, base); });
+    xfce4::connect_mode_changed    (plugin, [base](XfcePanelPlugin *p, XfcePanelPluginMode mode) { mode_cb (p, base); });
+    xfce4::connect_size_changed    (plugin, [base](XfcePanelPlugin *p, guint size) { return size_cb (p, size, base); });
 }
 
-static guint
-init_cpu_data (std::vector<CpuData> &data)
+static void
+init_cpu_data (const shared_ptr<CPUGraph> &base, bool read)
 {
-    guint cpuNr = detect_cpu_number ();
-    if (cpuNr != 0)
-        data.resize(cpuNr+1);
-    return cpuNr;
+    if (read)
+    {
+        read_cpu_data (base->cpu_data, base->cpu_to_index);
+
+        /* Read CPU data twice in order to initialize
+         * cpu_data[].previous_used and cpu_data[].previous_total
+         * with the current HWMs. HWM = High Water Mark. */
+        read_cpu_data (base->cpu_data, base->cpu_to_index);
+    }
+
+    base->nr_cores = base->cpu_to_index.size();
+
+    base->index_to_cpu.clear();
+    for (const auto [cpu, idx] : base->cpu_to_index)
+        base->index_to_cpu[idx] = cpu;
+    // CPU at index 0 (overall usage) has always index 0 - no need to map it:
+    // base->cpu_data[0] is always the same as base->cpu_data[base->index_to_cpu[0]]
+
+    if (base->nr_cores == 0)
+        fprintf (stderr, "Cannot init cpu data !\n");
+
+    base->topology = read_topology ();
 }
 
-static Ptr<CPUGraph>
+static shared_ptr<CPUGraph>
 create_gui (XfcePanelPlugin *plugin)
 {
     GtkWidget *frame, *ebox;
     GtkOrientation orientation;
-    auto base = xfce4::make<CPUGraph>();
+    auto base = make_shared<CPUGraph>();
 
     orientation = xfce_panel_plugin_get_orientation (plugin);
-    if ((base->nr_cores = init_cpu_data (base->cpu_data)) == 0)
-        fprintf (stderr,"Cannot init cpu data !\n");
 
-    /* Read CPU data twice in order to initialize
-     * cpu_data[].previous_used and cpu_data[].previous_total
-     * with the current HWMs. HWM = High Water Mark. */
-    read_cpu_data (base->cpu_data);
-    read_cpu_data (base->cpu_data);
-
-    base->topology = read_topology ();
+    init_cpu_data (base, true);
 
     base->plugin = plugin;
 
     base->ebox = ebox = gtk_event_box_new ();
-    gtk_event_box_set_visible_window (GTK_EVENT_BOX (ebox), FALSE);
-    gtk_event_box_set_above_child (GTK_EVENT_BOX (ebox), TRUE);
+    gtk_event_box_set_visible_window (GTK_EVENT_BOX (ebox), false);
+    gtk_event_box_set_above_child (GTK_EVENT_BOX (ebox), true);
     gtk_container_add (GTK_CONTAINER (plugin), ebox);
     xfce_panel_plugin_add_action_widget (plugin, ebox);
     xfce4::connect_button_press (ebox, [base](GtkWidget*, GdkEventButton *event) -> Propagation {
@@ -120,13 +132,13 @@ create_gui (XfcePanelPlugin *plugin)
 
     base->box = gtk_box_new (orientation, 0);
     gtk_container_add (GTK_CONTAINER (ebox), base->box);
-    gtk_widget_set_has_tooltip (base->box, TRUE);
+    gtk_widget_set_has_tooltip (base->box, true);
     xfce4::connect_query_tooltip (base->box, [base](GtkWidget *widget, gint x, gint y, bool keyboard, GtkTooltip *tooltip) {
         return tooltip_cb (tooltip, base);
     });
 
-    base->frame_widget = frame = gtk_frame_new (NULL);
-    gtk_box_pack_end (GTK_BOX (base->box), frame, TRUE, TRUE, 2);
+    base->frame_widget = frame = gtk_frame_new (nullptr);
+    gtk_box_pack_end (GTK_BOX (base->box), frame, true, true, 2);
 
     base->draw_area = gtk_drawing_area_new ();
     gtk_container_add (GTK_CONTAINER (frame), GTK_WIDGET (base->draw_area));
@@ -142,7 +154,8 @@ create_gui (XfcePanelPlugin *plugin)
     mode_cb (plugin, base);
     gtk_widget_show_all (ebox);
 
-    base->tooltip_text = gtk_label_new (NULL);
+    base->tooltip_last_value = -1;
+    base->tooltip_text = gtk_label_new (nullptr);
     g_object_ref (base->tooltip_text);
 
     return base;
@@ -161,10 +174,10 @@ about_cb ()
         "Jan Ziak <0xe2.0x9a.0x9b@xfce.org>",
         "Ludovic Mercier <lidiriel@coriolys.org>",
         "Peter Tribble <peter.tribble@gmail.com>",
-        NULL
+        nullptr
     };
 
-    gtk_show_about_dialog (NULL,
+    gtk_show_about_dialog (nullptr,
         "logo-icon-name", "org.xfce.panel.cpugraph",
         "license", xfce_get_license_text (XFCE_LICENSE_TEXT_GPL),
         "version", PACKAGE_VERSION,
@@ -172,41 +185,39 @@ about_cb ()
         "comments", _("Graphical representation of the CPU load"),
         "website", "https://docs.xfce.org/panel-plugins/xfce4-cpugraph-plugin",
         "copyright", "Copyright \302\251 2004-2023 The Xfce development team",
-        "authors", auth, NULL);
+        "authors", auth, nullptr);
 }
 
-static void
-ebox_revalidate (const Ptr<CPUGraph> &base)
+void
+CPUGraph::ebox_revalidate ()
 {
-    gtk_event_box_set_above_child (GTK_EVENT_BOX (base->ebox), FALSE);
-    gtk_event_box_set_above_child (GTK_EVENT_BOX (base->ebox), TRUE);
+    gtk_event_box_set_above_child (GTK_EVENT_BOX (ebox), false);
+    gtk_event_box_set_above_child (GTK_EVENT_BOX (ebox), true);
 }
 
-static guint
-nb_bars (const Ptr<const CPUGraph> &base)
+guint
+CPUGraph::nb_bars ()
 {
-    return base->tracked_core == 0 ? base->nr_cores : 1;
+    return tracked_core == 0 ? nr_cores : 1;
 }
 
-static void
-create_bars (const Ptr<CPUGraph> &base, GtkOrientation orientation)
+void
+CPUGraph::create_bars (GtkOrientation orientation)
 {
-    base->bars.frame = gtk_frame_new (NULL);
-    base->bars.draw_area = gtk_drawing_area_new ();
-    base->bars.orientation = orientation;
-    CPUGraph::set_frame (base, base->has_frame);
-    gtk_container_add (GTK_CONTAINER (base->bars.frame), base->bars.draw_area);
-    gtk_box_pack_end (GTK_BOX (base->box), base->bars.frame, TRUE, TRUE, 0);
-    xfce4::connect_after_draw (base->bars.draw_area, [base](cairo_t *cr) { return draw_bars_cb(cr, base); });
-    gtk_widget_show_all (base->bars.frame);
-    ebox_revalidate (base);
+    bars.frame = gtk_frame_new (nullptr);
+    bars.draw_area = gtk_drawing_area_new ();
+    bars.orientation = orientation;
+    set_frame (has_frame);
+    gtk_container_add (GTK_CONTAINER (bars.frame), bars.draw_area);
+    gtk_box_pack_end (GTK_BOX (box), bars.frame, true, true, 0);
+    xfce4::connect_after_draw (bars.draw_area, [base = shared_from_this ()](cairo_t *cr) { return draw_bars_cb(cr, base); });
+    gtk_widget_show_all (bars.frame);
+    ebox_revalidate ();
 }
 
-CPUGraph::~CPUGraph()
+CPUGraph::~CPUGraph ()
 {
     g_info ("%s", __PRETTY_FUNCTION__);
-    for (auto hist_data : history.data)
-        g_free (hist_data);
     if (channel)
     {
         g_object_unref (channel);
@@ -214,23 +225,36 @@ CPUGraph::~CPUGraph()
     }
 }
 
-static void
-shutdown (const Ptr<CPUGraph> &base)
+bool
+CPUGraph::is_smt_issues_enabled () const
 {
-    delete_bars (base);
+    return stats_smt || (has_bars && highlight_smt);
+}
+
+static void
+shutdown (const shared_ptr<CPUGraph> &base)
+{
+    base->delete_bars ();
     gtk_widget_destroy (base->ebox);
-    base->ebox = NULL;
+    base->ebox = nullptr;
     g_object_unref (base->tooltip_text);
-    base->tooltip_text = NULL;
-    if (base->timeout_id)
+    base->tooltip_text = nullptr;
+    xfce4::source_remove (base->timeout_id);
+}
+
+void
+CPUGraph::delete_bars ()
+{
+    if (bars.frame)
     {
-        g_source_remove (base->timeout_id);
-        base->timeout_id = 0;
+        gtk_widget_destroy (bars.frame);
+        bars.frame = nullptr;
+        bars.draw_area = nullptr;
     }
 }
 
 static void
-queue_draw (const Ptr<CPUGraph> &base)
+queue_draw (const shared_ptr<CPUGraph> &base)
 {
     if (base->mode != MODE_DISABLED)
         gtk_widget_queue_draw (base->draw_area);
@@ -239,18 +263,7 @@ queue_draw (const Ptr<CPUGraph> &base)
 }
 
 static void
-delete_bars (const Ptr<CPUGraph> &base)
-{
-    if (base->bars.frame)
-    {
-        gtk_widget_destroy (base->bars.frame);
-        base->bars.frame = NULL;
-        base->bars.draw_area = NULL;
-    }
-}
-
-static void
-resize_history (const Ptr<CPUGraph> &base, gssize history_size)
+resize_history (const shared_ptr<CPUGraph> &base, gssize history_size)
 {
     const guint fastest = get_update_interval_ms (RATE_FASTEST);
     const guint slowest = get_update_interval_ms (RATE_SLOWEST);
@@ -264,7 +277,7 @@ resize_history (const Ptr<CPUGraph> &base, gssize history_size)
 
     if (cap_pow2 != old_cap_pow2)
     {
-        const std::vector<CpuLoad*> old_data = std::move(base->history.data);
+        const auto old_data = move(base->history.data);
         const gssize old_mask = base->history.mask();
         const gssize old_offset = base->history.offset;
 
@@ -273,23 +286,24 @@ resize_history (const Ptr<CPUGraph> &base, gssize history_size)
         base->history.offset = 0;
         for (guint core = 0; core < base->nr_cores + 1; core++)
         {
-            base->history.data[core] = (CpuLoad*) g_malloc0 (cap_pow2 * sizeof (CpuLoad));
+            base->history.data[core] = make_unique<CpuLoad[]> (cap_pow2);
             if (!old_data.empty())
             {
                 for (gssize i = 0; i < old_cap_pow2 && i < cap_pow2; i++)
                     base->history.data[core][i] = old_data[core][(old_offset + i) & old_mask];
-                g_free (old_data[core]);
             }
         }
 
-        xfce4::trim_memory ();
+#ifdef HAVE_MALLOC_TRIM
+        malloc_trim (0);
+#endif
     }
 
     base->history.size = history_size;
 }
 
-static PluginSize
-size_cb (XfcePanelPlugin *plugin, guint plugin_size, const Ptr<CPUGraph> &base)
+static PluginShape
+size_cb (XfcePanelPlugin *plugin, guint plugin_size, const shared_ptr<CPUGraph> &base)
 {
     gint frame_h, frame_v, size;
     gssize history;
@@ -335,7 +349,7 @@ size_cb (XfcePanelPlugin *plugin, guint plugin_size, const Ptr<CPUGraph> &base)
     gtk_widget_set_size_request (GTK_WIDGET (base->frame_widget), frame_h, frame_v);
     if (base->has_bars) {
         base->bars.orientation = orientation;
-        set_bars_size (base);
+        base->set_bars_size ();
     }
 
     if (base->has_border)
@@ -344,35 +358,35 @@ size_cb (XfcePanelPlugin *plugin, guint plugin_size, const Ptr<CPUGraph> &base)
         border_width = 0;
     gtk_container_set_border_width (GTK_CONTAINER (base->box), border_width);
 
-    base->set_border (base, base->has_border);
+    base->set_border (base->has_border);
 
-    return xfce4::RECTANGLE;
+    return xfce4::PluginShape::Rectangle();
 }
 
-static void
-set_bars_size (const Ptr<CPUGraph> &base)
+void
+CPUGraph::set_bars_size ()
 {
     gint h, v;
     gint shadow_width;
 
-    shadow_width = base->has_frame ? 2*1 : 0;
+    shadow_width = has_frame ? 2*1 : 0;
 
-    if (base->bars.orientation == GTK_ORIENTATION_HORIZONTAL)
+    if (bars.orientation == GTK_ORIENTATION_HORIZONTAL)
     {
-        h = 6 * nb_bars (base) - 2 + shadow_width;
+        h = 6 * nb_bars () - 2 + shadow_width;
         v = -1;
     }
     else
     {
         h = -1;
-        v = 6 * nb_bars (base) - 2 + shadow_width;
+        v = 6 * nb_bars () - 2 + shadow_width;
     }
 
-    gtk_widget_set_size_request (base->bars.frame, h, v);
+    gtk_widget_set_size_request (bars.frame, h, v);
 }
 
 static void
-mode_cb (XfcePanelPlugin *plugin, const Ptr<CPUGraph> &base)
+mode_cb (XfcePanelPlugin *plugin, const shared_ptr<CPUGraph> &base)
 {
     gtk_orientable_set_orientation (GTK_ORIENTABLE (base->box), xfce_panel_plugin_get_orientation (plugin));
 
@@ -380,7 +394,7 @@ mode_cb (XfcePanelPlugin *plugin, const Ptr<CPUGraph> &base)
 }
 
 static void
-detect_smt_issues (const Ptr<CPUGraph> &base)
+detect_smt_issues (const shared_ptr<CPUGraph> &base)
 {
     const bool debug = false;
     gfloat actual_load[base->nr_cores];
@@ -389,7 +403,7 @@ detect_smt_issues (const Ptr<CPUGraph> &base)
 
     for (guint i = 0; i < base->nr_cores; i++)
     {
-        actual_load[i] = base->cpu_data[i+1].load;
+        actual_load[i] = base->cpu_data[base->index_to_cpu[i+1]].load;
         suboptimal[i] = false;
         movement[i] = false;
         if (debug)
@@ -398,11 +412,7 @@ detect_smt_issues (const Ptr<CPUGraph> &base)
 
     if (G_LIKELY (base->topology && base->topology->smt))
     {
-        /* Use <Topology> instead of <const Topology>.
-         * The non-const version results in less efficient C++ code,
-         * but it is less prone to generate an exception or a crash
-         * than the const version due to an unforseen programming bug. */
-        const Ptr0<Topology> topo = base->topology;
+        const auto topo = base->topology.get();
 
         gfloat optimal_load[base->nr_cores];
         gfloat actual_num_instr_executed[base->nr_cores];
@@ -615,16 +625,61 @@ detect_smt_issues (const Ptr<CPUGraph> &base)
     }
 
     for (guint i = 0; i < base->nr_cores; i++)
-        base->cpu_data[i+1].smt_highlight = suboptimal[i];
+        base->cpu_data[base->index_to_cpu[i+1]].smt_highlight = suboptimal[i];
 }
 
 static xfce4::TimeoutResponse
-update_cb (const Ptr<CPUGraph> &base)
+update_cb (const shared_ptr<CPUGraph> &base)
 {
-    if (!read_cpu_data (base->cpu_data))
-        return xfce4::TIMEOUT_AGAIN;
+    read_cpu_data (base->cpu_data, base->cpu_to_index_cache);
 
-    if (base->topology && base->topology->smt && base->isSmtIssuesEnabled ())
+    if (base->cpu_to_index_cache.empty()) // Read Error
+    {
+        return xfce4::TimeoutResponse::Again();
+    }
+    else if (base->cpu_to_index_cache != base->cpu_to_index) // CPU layout changed
+    {
+        const auto old_cpus = move (base->cpu_to_index);
+
+        // Init CPU data
+        base->cpu_to_index = move (base->cpu_to_index_cache);
+        init_cpu_data (base, false);
+
+        if (!base->history.data.empty () && old_cpus != base->cpu_to_index)
+        {
+            // There are changes in CPU layout - (reorder / allocate new / free old) per-core history
+
+            auto old_history_data = move (base->history.data);
+
+            base->history.data.resize (base->nr_cores + 1);
+
+            // Copy old history pointer for overall CPU usage
+            base->history.data[0] = move(old_history_data[0]);
+
+            // Copy old history pointers to the new CPU layout with the new CPUs order
+            for (const auto [cpu, idx] : base->cpu_to_index)
+            {
+                const auto old_it = old_cpus.find (cpu);
+                if (old_it == old_cpus.end ())
+                    continue; // CPU disappeared, ignore
+
+                const auto old_idx = old_it->second;
+                base->history.data[idx] = move(old_history_data[old_idx]);
+            }
+
+            // Allocate memory for all new CPUs
+            for (auto &&data : base->history.data)
+            {
+                if (!data)
+                    data = make_unique<CpuLoad[]> (base->history.cap_pow2);
+            }
+        }
+
+        // Update GUI and init history if necessary
+        size_cb (base->plugin, xfce_panel_plugin_get_size (base->plugin), base);
+    }
+
+    if (base->topology && base->topology->smt && base->is_smt_issues_enabled ())
         detect_smt_issues (base);
 
     if (!base->history.data.empty())
@@ -635,7 +690,7 @@ update_cb (const Ptr<CPUGraph> &base)
         base->history.offset = (base->history.offset - 1) & base->history.mask();
         for (guint core = 0; core < base->nr_cores + 1; core++)
         {
-            const CpuData &cpu_core_data = base->cpu_data[core];
+            const CpuData &cpu_core_data = base->cpu_data[base->index_to_cpu[core]];
             CpuLoad &load = base->history.data[core][base->history.offset];
             load.timestamp = timestamp;
             load.value = cpu_core_data.load;
@@ -647,32 +702,37 @@ update_cb (const Ptr<CPUGraph> &base)
     }
 
     queue_draw (base);
-    update_tooltip (base);
+    update_tooltip (base, false);
 
-    return xfce4::TIMEOUT_AGAIN;
+    return xfce4::TimeoutResponse::Again();
 }
 
 static void
-update_tooltip (const Ptr<CPUGraph> &base)
+update_tooltip (const shared_ptr<CPUGraph> &base, bool force)
 {
-    auto tooltip = xfce4::sprintf (_("CPU usage: %.1f%%"), base->cpu_data[0].load * 100.0f);
-    if (gtk_label_get_text (GTK_LABEL (base->tooltip_text)) != tooltip)
+    const gint value = round(base->cpu_data[0].load * 1000.0f);
+    if (base->tooltip_last_value != value && (force || gtk_widget_get_mapped (base->tooltip_text)))
+    {
+        auto tooltip = xfce4::sprintf (_("CPU usage: %.1f%%"), value / 10.0f);
         gtk_label_set_text (GTK_LABEL (base->tooltip_text), tooltip.c_str());
+        base->tooltip_last_value = value;
+    }
 }
 
 static TooltipTime
-tooltip_cb (GtkTooltip *tooltip, const Ptr<CPUGraph> &base)
+tooltip_cb (GtkTooltip *tooltip, const shared_ptr<CPUGraph> &base)
 {
+    update_tooltip (base, true);
     gtk_tooltip_set_custom (tooltip, base->tooltip_text);
-    return xfce4::NOW;
+    return xfce4::TooltipTime::Now();
 }
 
 static Propagation
-draw_area_cb (cairo_t *cr, const Ptr<CPUGraph> &base)
+draw_area_cb (cairo_t *cr, const shared_ptr<CPUGraph> &base)
 {
     GtkAllocation alloc;
     gint w, h;
-    void (*draw) (const Ptr<CPUGraph> &base, cairo_t *cr, gint w, gint h, guint core) = NULL;
+    void (*draw) (const shared_ptr<CPUGraph> &base, cairo_t *cr, gint w, gint h, guint core) = nullptr;
 
     gtk_widget_get_allocation (base->draw_area, &alloc);
     w = alloc.width;
@@ -683,7 +743,10 @@ draw_area_cb (cairo_t *cr, const Ptr<CPUGraph> &base)
         case MODE_DISABLED:
             break;
         case MODE_NORMAL:
-            draw = draw_graph_normal;
+            if (base->size > 1)
+                draw = draw_graph_normal;
+            else
+                draw = draw_graph_no_history;
             break;
         case MODE_LED:
             draw = draw_graph_LED;
@@ -702,9 +765,9 @@ draw_area_cb (cairo_t *cr, const Ptr<CPUGraph> &base)
         {
             guint core;
 
-            if (!base->colors[BG_COLOR].isTransparent())
+            if (!base->colors[BG_COLOR].is_transparent())
             {
-                xfce4::cairo_set_source (cr, base->colors[BG_COLOR]);
+                xfce4::cairo_set_source_rgba (cr, base->colors[BG_COLOR]);
                 cairo_rectangle (cr, 0, 0, w, h);
                 cairo_fill (cr);
             }
@@ -739,9 +802,9 @@ draw_area_cb (cairo_t *cr, const Ptr<CPUGraph> &base)
                     *(horizontal ? &translation.x : &translation.y) = core * (base->size + base->per_core_spacing);
                     cairo_translate (cr, translation.x, translation.y);
 
-                    if (!base->colors[BG_COLOR].isTransparent())
+                    if (!base->colors[BG_COLOR].is_transparent())
                     {
-                        xfce4::cairo_set_source (cr, base->colors[BG_COLOR]);
+                        xfce4::cairo_set_source_rgba (cr, base->colors[BG_COLOR]);
                         cairo_rectangle (cr, 0, 0, w1, h1);
                         cairo_fill (cr);
                     }
@@ -754,11 +817,11 @@ draw_area_cb (cairo_t *cr, const Ptr<CPUGraph> &base)
             }
         }
     }
-    return xfce4::PROPAGATE;
+    return xfce4::Propagation::Propagate();
 }
 
 static Propagation
-draw_bars_cb (cairo_t *cr, const Ptr<CPUGraph> &base)
+draw_bars_cb (cairo_t *cr, const shared_ptr<CPUGraph> &base)
 {
     GtkAllocation alloc;
     gfloat size;
@@ -766,9 +829,9 @@ draw_bars_cb (cairo_t *cr, const Ptr<CPUGraph> &base)
 
     gtk_widget_get_allocation (base->bars.draw_area, &alloc);
 
-    if (!base->colors[BG_COLOR].isTransparent())
+    if (!base->colors[BG_COLOR].is_transparent())
     {
-        xfce4::cairo_set_source (cr, base->colors[BG_COLOR]);
+        xfce4::cairo_set_source_rgba (cr, base->colors[BG_COLOR]);
         cairo_rectangle (cr, 0, 0, alloc.width, alloc.height);
         cairo_fill (cr);
     }
@@ -781,7 +844,7 @@ draw_bars_cb (cairo_t *cr, const Ptr<CPUGraph> &base)
             usage = 0;
         usage *= size;
 
-        xfce4::cairo_set_source (cr, base->colors[BARS_COLOR]);
+        xfce4::cairo_set_source_rgba (cr, base->colors[BARS_COLOR]);
         if (horizontal)
             cairo_rectangle (cr, 0, size-usage, 4, usage);
         else
@@ -790,13 +853,15 @@ draw_bars_cb (cairo_t *cr, const Ptr<CPUGraph> &base)
     }
     else
     {
-        const xfce4::RGBA *active_color = NULL;
+        const xfce4::RGBA *active_color = nullptr;
         bool fill = false;
         for (guint i = 0; i < base->nr_cores; i++)
         {
-            const bool highlight = base->highlight_smt && base->cpu_data[i+1].smt_highlight;
+            const auto &cpu_data_i_plus_1 = base->cpu_data[base->index_to_cpu[i+1]];
 
-            gfloat usage = base->cpu_data[i+1].load;
+            const bool highlight = base->highlight_smt && cpu_data_i_plus_1.smt_highlight;
+
+            gfloat usage = cpu_data_i_plus_1.load;
             if (usage < base->load_threshold)
                 usage = 0;
             usage *= size;
@@ -810,7 +875,7 @@ draw_bars_cb (cairo_t *cr, const Ptr<CPUGraph> &base)
                     cairo_fill (cr);
                     fill = false;
                 }
-                xfce4::cairo_set_source (cr, *color);
+                xfce4::cairo_set_source_rgba (cr, *color);
                 active_color = color;
             }
 
@@ -823,7 +888,7 @@ draw_bars_cb (cairo_t *cr, const Ptr<CPUGraph> &base)
         if (fill)
             cairo_fill (cr);
     }
-    return xfce4::PROPAGATE;
+    return xfce4::Propagation::Propagate();
 }
 
 static const gchar*
@@ -856,11 +921,11 @@ default_command (bool *in_terminal, bool *startup_notification)
 }
 
 static Propagation
-command_cb (GdkEventButton *event, const Ptr<CPUGraph> &base)
+command_cb (GdkEventButton *event, const shared_ptr<CPUGraph> &base)
 {
     if (event->button == 1)
     {
-        std::string command;
+        string command;
         bool in_terminal, startup_notification;
 
         if (!base->command.empty())
@@ -874,11 +939,11 @@ command_cb (GdkEventButton *event, const Ptr<CPUGraph> &base)
             command = default_command (&in_terminal, &startup_notification);
         }
 
-        xfce_spawn_command_line_on_screen (gdk_screen_get_default (),
-                                           command.c_str(), in_terminal,
-                                           startup_notification, NULL);
+        xfce_spawn_command_line (gdk_screen_get_default (),
+                                 command.c_str(), in_terminal,
+                                 startup_notification, true, nullptr);
     }
-    return xfce4::STOP;
+    return xfce4::Propagation::Stop();
 }
 
 /**
@@ -907,83 +972,85 @@ get_update_interval_ms (CPUGraphUpdateRate rate)
 }
 
 void
-CPUGraph::set_startup_notification (const Ptr<CPUGraph> &base, bool startup_notification)
+CPUGraph::set_startup_notification (bool startup_notification)
 {
-    base->command_startup_notification = startup_notification;
+    command_startup_notification = startup_notification;
 }
 
 void
-CPUGraph::set_in_terminal (const Ptr<CPUGraph> &base, bool in_terminal)
+CPUGraph::set_in_terminal (bool in_terminal)
 {
-    base->command_in_terminal = in_terminal;
+    command_in_terminal = in_terminal;
 }
 
 void
-CPUGraph::set_command (const Ptr<CPUGraph> &base, const std::string &command)
+CPUGraph::set_command (const string_view &command_arg)
 {
-    base->command = xfce4::trim (command);
+    command = xfce4::trim (command_arg);
 }
 
 void
-CPUGraph::set_bars (const Ptr<CPUGraph> &base, bool has_bars)
+CPUGraph::set_bars (bool has_bars_arg)
 {
-    if (base->has_bars != has_bars)
+    if (has_bars != has_bars_arg)
     {
-        base->has_bars = has_bars;
-        if (base->has_bars)
+        has_bars = has_bars_arg;
+        if (has_bars)
         {
-            create_bars (base, xfce_panel_plugin_get_orientation (base->plugin));
-            set_bars_size (base);
+            create_bars (xfce_panel_plugin_get_orientation (plugin));
+            set_bars_size ();
         }
         else
-            delete_bars (base);
+        {
+            delete_bars ();
+        }
     }
 }
 
 void
-CPUGraph::set_border (const Ptr<CPUGraph> &base, bool has_border)
+CPUGraph::set_border (bool has_border_arg)
 {
-    if (base->has_border != has_border)
+    if (has_border != has_border_arg)
     {
-        base->has_border = has_border;
-        size_cb (base->plugin, xfce_panel_plugin_get_size (base->plugin), base);
+        has_border = has_border_arg;
+        size_cb (plugin, xfce_panel_plugin_get_size (plugin), shared_from_this ());
     }
 }
 
 void
-CPUGraph::set_frame (const Ptr<CPUGraph> &base, bool has_frame)
+CPUGraph::set_frame (bool has_frame_arg)
 {
-    base->has_frame = has_frame;
-    gtk_frame_set_shadow_type (GTK_FRAME (base->frame_widget), has_frame ? GTK_SHADOW_IN : GTK_SHADOW_NONE);
-    if (base->bars.frame)
-        gtk_frame_set_shadow_type (GTK_FRAME (base->bars.frame), has_frame ? GTK_SHADOW_IN : GTK_SHADOW_NONE);
-    size_cb (base->plugin, xfce_panel_plugin_get_size (base->plugin), base);
+    has_frame = has_frame_arg;
+    gtk_frame_set_shadow_type (GTK_FRAME (frame_widget), has_frame ? GTK_SHADOW_IN : GTK_SHADOW_NONE);
+    if (bars.frame)
+        gtk_frame_set_shadow_type (GTK_FRAME (bars.frame), has_frame ? GTK_SHADOW_IN : GTK_SHADOW_NONE);
+    size_cb (plugin, xfce_panel_plugin_get_size (plugin), shared_from_this ());
 }
 
 void
-CPUGraph::set_nonlinear_time (const Ptr<CPUGraph> &base, bool non_linear)
+CPUGraph::set_nonlinear_time (bool non_linear_arg)
 {
-    if (base->non_linear != non_linear)
+    if (non_linear != non_linear_arg)
     {
-        base->non_linear = non_linear;
+        non_linear = non_linear_arg;
         if (!non_linear)
-            base->non_linear_cache = {};
-        queue_draw (base);
+            non_linear_cache = {};
+        queue_draw (shared_from_this ());
     }
 }
 
 void
-CPUGraph::set_per_core (const Ptr<CPUGraph> &base, bool per_core)
+CPUGraph::set_per_core (bool per_core_arg)
 {
-    if (base->per_core != per_core)
+    if (per_core != per_core_arg)
     {
-        base->per_core = per_core;
-        size_cb (base->plugin, xfce_panel_plugin_get_size (base->plugin), base);
+        per_core = per_core_arg;
+        size_cb (plugin, xfce_panel_plugin_get_size (plugin), shared_from_this ());
     }
 }
 
 void
-CPUGraph::set_per_core_spacing (const Ptr<CPUGraph> &base, guint spacing)
+CPUGraph::set_per_core_spacing (guint spacing)
 {
     /* Use <=, instead of <, supresses a compiler warning */
     if (G_UNLIKELY (spacing <= PER_CORE_SPACING_MIN))
@@ -991,124 +1058,123 @@ CPUGraph::set_per_core_spacing (const Ptr<CPUGraph> &base, guint spacing)
     if (G_UNLIKELY (spacing > PER_CORE_SPACING_MAX))
         spacing = PER_CORE_SPACING_MAX;
 
-    if (base->per_core_spacing != spacing)
+    if (per_core_spacing != spacing)
     {
-        base->per_core_spacing = spacing;
-        size_cb (base->plugin, xfce_panel_plugin_get_size (base->plugin), base);
+        per_core_spacing = spacing;
+        size_cb (plugin, xfce_panel_plugin_get_size (plugin), shared_from_this ());
     }
 }
 
 void
-CPUGraph::set_stats_smt (const Ptr<CPUGraph> &base, bool stats_smt)
+CPUGraph::set_stats_smt (bool stats_smt_arg)
 {
-    base->stats_smt = stats_smt;
+    stats_smt = stats_smt_arg;
 }
 
 void
-CPUGraph::set_smt (const Ptr<CPUGraph> &base, bool highlight_smt)
+CPUGraph::set_smt (bool highlight_smt_arg)
 {
-    base->highlight_smt = highlight_smt;
+    highlight_smt = highlight_smt_arg;
 }
 
 void
-CPUGraph::set_update_rate (const Ptr<CPUGraph> &base, CPUGraphUpdateRate rate)
+CPUGraph::set_update_rate (CPUGraphUpdateRate rate)
 {
-    bool change = (base->update_interval != rate);
-    bool init = (base->timeout_id == 0);
+    bool change = (update_interval != rate);
+    bool init = timeout_id.expired();
 
     if (change || init)
     {
         guint interval = get_update_interval_ms (rate);
 
-        base->update_interval = rate;
-        if (base->timeout_id)
-            g_source_remove (base->timeout_id);
-        base->timeout_id = xfce4::timeout_add (interval, [base]() { return update_cb(base); });
+        update_interval = rate;
+        xfce4::source_remove (timeout_id);
+        timeout_id = xfce4::timeout_add (interval, [base = shared_from_this ()]() { return update_cb (base); });
 
         if (change && !init)
-            queue_draw (base);
+            queue_draw (shared_from_this ());
     }
 }
 
 void
-CPUGraph::maybe_clear_smt_stats (const Ptr<CPUGraph> &base)
+CPUGraph::maybe_clear_smt_stats ()
 {
-    if (!base->isSmtIssuesEnabled ())
-        base->stats = {};
+    if (!is_smt_issues_enabled ())
+        stats = {};
 }
 
 void
-CPUGraph::set_size (const Ptr<CPUGraph> &base, guint size)
+CPUGraph::set_size (guint size_arg)
 {
     if (G_UNLIKELY (size < MIN_SIZE))
         size = MIN_SIZE;
     if (G_UNLIKELY (size > MAX_SIZE))
         size = MAX_SIZE;
 
-    base->size = size;
-    size_cb (base->plugin, xfce_panel_plugin_get_size (base->plugin), base);
+    size = size_arg;
+    size_cb (plugin, xfce_panel_plugin_get_size (plugin), shared_from_this ());
 }
 
 void
-CPUGraph::set_color_mode (const Ptr<CPUGraph> &base, guint color_mode)
+CPUGraph::set_color_mode (guint color_mode_arg)
 {
-    if (base->color_mode != color_mode)
+    if (color_mode != color_mode_arg)
     {
-        base->color_mode = color_mode;
-        queue_draw (base);
+        color_mode = color_mode_arg;
+        queue_draw (shared_from_this ());
     }
 }
 
 void
-CPUGraph::set_mode (const Ptr<CPUGraph> &base, CPUGraphMode mode)
+CPUGraph::set_mode (CPUGraphMode mode_arg)
 {
-    base->mode = mode;
-    base->nearest_cache = {};
-    base->non_linear_cache = {};
+    mode = mode_arg;
+    nearest_cache = {};
+    non_linear_cache = {};
     if (mode == MODE_DISABLED)
     {
-        gtk_widget_hide (base->frame_widget);
+        gtk_widget_hide (frame_widget);
     }
     else
     {
-        gtk_widget_show (base->frame_widget);
-        ebox_revalidate (base);
+        gtk_widget_show (frame_widget);
+        ebox_revalidate ();
     }
 }
 
 void
-CPUGraph::set_color (const Ptr<CPUGraph> &base, CPUGraphColorNumber number, const xfce4::RGBA &color)
+CPUGraph::set_color (CPUGraphColorNumber number, const xfce4::RGBA &color)
 {
-    if (!base->colors[number].equals(color))
+    if (colors[number] != color)
     {
-        base->colors[number] = color;
-        queue_draw (base);
+        colors[number] = color;
+        queue_draw (shared_from_this ());
     }
 }
 
 void
-CPUGraph::set_tracked_core (const Ptr<CPUGraph> &base, guint core)
+CPUGraph::set_tracked_core (guint core)
 {
-    if (G_UNLIKELY (core > base->nr_cores + 1))
+    if (G_UNLIKELY (core > nr_cores + 1))
         core = 0;
 
-    if (base->tracked_core != core)
+    if (tracked_core != core)
     {
-        const bool had_bars = base->has_bars;
+        const bool had_bars = has_bars;
         if (had_bars)
-            set_bars (base, false);
-        base->tracked_core = core;
+            set_bars (false);
+        tracked_core = core;
         if (had_bars)
-            set_bars (base, true);
+            set_bars (true);
     }
 }
 
 void
-CPUGraph::set_load_threshold (const Ptr<CPUGraph> &base, gfloat threshold)
+CPUGraph::set_load_threshold (gfloat threshold)
 {
     if (threshold < 0)
         threshold = 0;
     if (threshold > MAX_LOAD_THRESHOLD)
         threshold = MAX_LOAD_THRESHOLD;
-    base->load_threshold = threshold;
+    load_threshold = threshold;
 }
