@@ -29,6 +29,8 @@
 #endif
 
 #include "os.h"
+#include "../xfce4cpp/string-utils.hh"
+#include "../xfce4cpp/io.hh"
 
 #include <errno.h>
 #include <stdio.h>
@@ -38,7 +40,7 @@
 #include <unordered_set>
 #include <vector>
 
-using xfce4::parse_ulong;
+using namespace std;
 
 #if defined (__linux__) || defined (__FreeBSD_kernel__)
 #define PROC_STAT "/proc/stat"
@@ -77,43 +79,17 @@ static kstat_ctl_t *kc;
 #endif
 
 #if defined (__linux__) || defined (__FreeBSD_kernel__)
-guint
-detect_cpu_number ()
+void
+read_cpu_data (unordered_map<guint, CpuData> &data, unordered_map<guint, guint> &cpu_to_index)
 {
-    FILE *fstat = NULL;
-    if (!(fstat = fopen (PROC_STAT, "r")))
-        return 0;
-
-    guint nb_cpu = 0;
-    gchar cpuStr[PROCMAXLNLEN];
-    while (fgets (cpuStr, PROCMAXLNLEN, fstat))
-    {
-        if (strncmp (cpuStr, "cpu", 3) != 0)
-            break;
-
-        gchar *s = cpuStr + 3;
-        if (!g_ascii_isspace (*s))
-        {
-            gulong cpu = parse_ulong(&s);
-            nb_cpu = MAX(nb_cpu, cpu + 1);
-        }
-    }
-
-    fclose (fstat);
-    return nb_cpu;
-}
-
-bool
-read_cpu_data (std::vector<CpuData> &data)
-{
-    if (G_UNLIKELY(data.size() == 0))
-        return false;
-
-    const size_t nb_cpu = data.size(); /* Number of CPU threads + 1 (index 0 - overall usage) */
     FILE *fStat;
 
+    cpu_to_index.clear();
+
     if (!(fStat = fopen (PROC_STAT, "r")))
-        return false;
+        return;
+
+    guint cpu_it = 0;
 
     while (true)
     {
@@ -121,7 +97,8 @@ read_cpu_data (std::vector<CpuData> &data)
         if (!fgets (cpuStr, PROCMAXLNLEN, fStat))
         {
             fclose (fStat);
-            return false;
+            cpu_to_index.clear();
+            return;
         }
 
         if (strncmp (cpuStr, "cpu", 3) != 0)
@@ -131,63 +108,65 @@ read_cpu_data (std::vector<CpuData> &data)
 
         guint cpu;
         if (g_ascii_isspace (*s))
-            cpu = 0;
-        else
-            cpu = 1 + parse_ulong (&s);
-
-        gulong user = parse_ulong (&s);
-        gulong nice = parse_ulong (&s);
-        gulong system = parse_ulong (&s);
-        gulong idle = parse_ulong (&s);
-        gulong iowait = parse_ulong (&s);
-        gulong irq = parse_ulong (&s);
-        gulong softirq = parse_ulong (&s);
-
-        if (G_LIKELY (cpu < nb_cpu))
         {
-            system += irq + softirq;
-            gulong total = system + user + nice + iowait + idle;
-
-            const bool total_greater = total > data[cpu].previous_total;
-            const gfloat divider = (gfloat) (total - data[cpu].previous_total);
-
-            if (total_greater && system >= data[cpu].previous_system)
-                data[cpu].system = (system - data[cpu].previous_system) / divider;
-            else
-                data[cpu].system = 0.0f;
-
-            if (total_greater && user >= data[cpu].previous_user)
-                data[cpu].user = (user - data[cpu].previous_user) / divider;
-            else
-                data[cpu].user = 0.0f;
-
-            if (total_greater && nice >= data[cpu].previous_nice)
-                data[cpu].nice = (nice - data[cpu].previous_nice) / divider;
-            else
-                data[cpu].nice = 0.0f;
-
-            if (total_greater && iowait >= data[cpu].previous_iowait)
-                data[cpu].iowait = (iowait - data[cpu].previous_iowait) / divider;
-            else
-                data[cpu].iowait = 0.0f;
-
-            data[cpu].load = data[cpu].user + data[cpu].system + data[cpu].nice;
-
-            data[cpu].previous_system = system;
-            data[cpu].previous_user = user;
-            data[cpu].previous_nice = nice;
-            data[cpu].previous_iowait = iowait;
-            data[cpu].previous_total = total;
+            cpu = 0;
         }
+        else
+        {
+            cpu = g_ascii_strtoull (s, &s, 0) + 1;
+            cpu_to_index[cpu] = ++cpu_it; // Map logical system CPU to array index
+        }
+
+        gulong user = g_ascii_strtoull (s, &s, 0);
+        gulong nice = g_ascii_strtoull (s, &s, 0);
+        gulong system = g_ascii_strtoull (s, &s, 0);
+        gulong idle = g_ascii_strtoull (s, &s, 0);
+        gulong iowait = g_ascii_strtoull (s, &s, 0);
+        gulong irq = g_ascii_strtoull (s, &s, 0);
+        gulong softirq = g_ascii_strtoull (s, &s, 0);
+
+        auto &data_cpu = data[cpu];
+
+        system += irq + softirq;
+        gulong total = system + user + nice + iowait + idle;
+
+        const bool total_greater = total > data_cpu.previous_total;
+        const gfloat divider = (gfloat) (total - data_cpu.previous_total);
+
+        if (total_greater && system >= data_cpu.previous_system)
+            data_cpu.system = (system - data_cpu.previous_system) / divider;
+        else
+            data_cpu.system = 0.0f;
+
+        if (total_greater && user >= data_cpu.previous_user)
+            data_cpu.user = (user - data_cpu.previous_user) / divider;
+        else
+            data_cpu.user = 0.0f;
+
+        if (total_greater && nice >= data_cpu.previous_nice)
+            data_cpu.nice = (nice - data_cpu.previous_nice) / divider;
+        else
+            data_cpu.nice = 0.0f;
+
+        if (total_greater && iowait >= data_cpu.previous_iowait)
+            data_cpu.iowait = (iowait - data_cpu.previous_iowait) / divider;
+        else
+            data_cpu.iowait = 0.0f;
+
+        data_cpu.load = data_cpu.user + data_cpu.system + data_cpu.nice;
+
+        data_cpu.previous_system = system;
+        data_cpu.previous_user = user;
+        data_cpu.previous_nice = nice;
+        data_cpu.previous_iowait = iowait;
+        data_cpu.previous_total = total;
     }
 
     fclose (fStat);
-
-    return true;
 }
 
 #elif defined (__FreeBSD__)
-guint
+static guint
 detect_cpu_number ()
 {
     static gint mib[] = {CTL_HW, HW_NCPU};
@@ -200,8 +179,8 @@ detect_cpu_number ()
         return ncpu;
 }
 
-bool
-read_cpu_data (std::vector<CpuData> &data)
+static bool
+read_cpu_data (unordered_map<guint, CpuData> &data)
 {
     if (G_UNLIKELY(data.size() == 0))
         return false;
@@ -252,7 +231,7 @@ read_cpu_data (std::vector<CpuData> &data)
 }
 
 #elif defined (__NetBSD__)
-guint
+static guint
 detect_cpu_number ()
 {
     static gint mib[] = {CTL_HW, HW_NCPU};
@@ -265,8 +244,8 @@ detect_cpu_number ()
         return ncpu;
 }
 
-bool
-read_cpu_data (std::vector<CpuData> &data)
+static bool
+read_cpu_data (unordered_map<guint, CpuData> &data)
 {
     if (G_UNLIKELY(data.size() == 0))
         return false;
@@ -302,7 +281,7 @@ read_cpu_data (std::vector<CpuData> &data)
 }
 
 #elif defined (__OpenBSD__)
-guint
+static guint
 detect_cpu_number ()
 {
     static gint mib[] = {CTL_HW, HW_NCPU};
@@ -315,8 +294,8 @@ detect_cpu_number ()
         return ncpu;
 }
 
-bool
-read_cpu_data (std::vector<CpuData> &data)
+static bool
+read_cpu_data (unordered_map<guint, CpuData> &data)
 {
     if (G_UNLIKELY(data.size() == 0))
         return false;
@@ -358,7 +337,7 @@ init_stats ()
     kc = kstat_open ();
 }
 
-guint
+static guint
 detect_cpu_number ()
 {
     kstat_t *ksp;
@@ -376,8 +355,8 @@ detect_cpu_number ()
     return knp->value.ui32;
 }
 
-bool
-read_cpu_data (std::vector<CpuData> &data)
+static bool
+read_cpu_data (unordered_map<guint, CpuData> &data)
 {
     if (G_UNLIKELY(data.size() == 0))
         return false;
@@ -427,12 +406,45 @@ read_cpu_data (std::vector<CpuData> &data)
 #endif
 
 
+#if !(defined (__linux__) || defined (__FreeBSD_kernel__))
+void
+read_cpu_data (unordered_map<guint, CpuData> &data, unordered_map<guint, guint> &cpu_to_index)
+{
+    static guint n_cpus = detect_cpu_number ();
 
-static Ptr0<Topology>
+    if (data.empty())
+    {
+        data.reserve (n_cpus + 1);
+        for (guint i = 0; i < n_cpus + 1; ++i)
+        {
+            data[i] = CpuData ();
+        }
+    }
+
+    if (read_cpu_data (data))
+    {
+        if (cpu_to_index.empty ())
+        {
+            cpu_to_index.reserve (n_cpus);
+            for (guint i = 0; i < n_cpus; ++i)
+            {
+                cpu_to_index[i] = i;
+            }
+        }
+    }
+    else
+    {
+        cpu_to_index.clear();
+    }
+}
+#endif
+
+
+static unique_ptr<Topology>
 read_topology_linux ()
 {
-    std::unordered_set<gint> core_ids;
-    std::unordered_map<guint, gint> logical_cpu_2_core;
+    unordered_set<gint> core_ids;
+    unordered_map<guint, gint> logical_cpu_2_core;
     gint max_core_id = -1;
 
     guint num_online_logical_cpus = 0;
@@ -445,26 +457,22 @@ read_topology_linux ()
         if (!xfce4::is_directory (xfce4::sprintf ("%s/cpu%d", sysfs_base, logical_cpu)))
             break;
 
-        std::string file_contents;
+        xfce4::g_string_view file_contents;
         if (xfce4::read_file (xfce4::sprintf ("%s/cpu%d/topology/core_id", sysfs_base, logical_cpu), file_contents))
         {
-            auto core_id_opt = xfce4::parse_long (file_contents, 10);
-            if (core_id_opt.has_value())
+            const auto core_id = xfce4::parse_long (file_contents, 10);
+            if (core_id.has_value() && G_LIKELY (core_id.value() >= 0 && core_id.value() <= G_MAXINT))
             {
-                auto core_id = core_id_opt.value();
-                if (G_LIKELY (core_id >= 0 && core_id <= G_MAXINT))
-                {
-                    num_online_logical_cpus++;
-                    core_ids.insert(core_id);
-                    logical_cpu_2_core[logical_cpu] = core_id;
-                    if (max_core_id < core_id)
-                        max_core_id = core_id;
-                }
-                else
-                    return nullptr;
+                num_online_logical_cpus++;
+                core_ids.insert(core_id.value());
+                logical_cpu_2_core[logical_cpu] = core_id.value();
+                if (max_core_id < core_id)
+                    max_core_id = core_id.value();
             }
             else
+            {
                 return nullptr;
+            }
         }
         else
         {
@@ -482,7 +490,7 @@ read_topology_linux ()
 
     if (!logical_cpu_2_core.empty())
     {
-        auto t = xfce4::make<Topology>();
+        auto t = make_unique<Topology>();
 
         /* Fill-in the topology data */
 
@@ -530,7 +538,7 @@ read_topology_linux ()
         return nullptr;
 }
 
-Ptr0<Topology>
+unique_ptr<Topology>
 read_topology ()
 {
     bool is_linux = false;
